import plistlib
import sys

import biplist
from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv

from artifacts.Artifact import Artifact


class SafariRecentWebSearches(ab.Artifact):
    _name = 'Safari Recent WebSearches'
    _search_dirs = '**/Library/Preferences/com.apple.mobilesafari.plist'
    _category = 'Safari Browser'

    def get(self, files_found, seeker):
        data_list = []
        for file_found in files_found:
            file_found = str(file_found)
            with open(file_found, "rb") as fp:
                try:
                    if sys.version_info >= (3, 9):
                        plist = plistlib.load(fp)
                    else:
                        plist = biplist.readPlist(fp)
                    searches = plist.get('RecentWebSearches', [])
                    for search in searches:
                        term = search.get('SearchString', '')
                        date = search.get('Date', '')
                        data_list.append((date, term))
                except (
                    biplist.InvalidPlistException,
                    plistlib.InvalidFileException,
                ) as ex:
                    logfunc(f'Failed to read plist {file_found} ' + str(ex))

        if data_list:
            report = ArtifactHtmlReport('Safari Recent WebSearches')
            report.start_artifact_report(report_folder, 'Recent WebSearches')
            report.add_script()
            data_headers = ('Date', 'Search Term')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Recent WebSearches'
            tsv(report_folder, data_headers, data_list, tsvname)
            timeline(report_folder, tsvname, data_list, data_headers)
        else:
            logfunc('No data for recent web searches')



from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import kmlgen, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class RoutineDCloud(ab.Artifact):
    _name = 'RoutineD Cloud'
    _search_dirs = '**/Library/Caches/com.apple.routined/Cloud-V2.sqlite*'
    _category = 'Locations'

    def get(self, files_found, seeker):
        iOSversion = artifacts.artGlobals.versionf
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('Cloud-V2.sqlite'):
                break

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zrtaddressmo.zcreationdate + 978307200, 'unixepoch'),
            datetime(zrtaddressmo.zexpirationdate + 978307200, 'unixepoch'),
            zrtaddressmo.zcountry,
            zrtaddressmo.zcountrycode,
            zrtaddressmo.zpostalcode,
            zrtaddressmo.zlocality,
            zrtaddressmo.zsublocality,
            zrtaddressmo.zthoroughfare,
            zrtaddressmo.zsubthoroughfare,
            zrtaddressmo.zsubadministrativearea,
            zrtaddressmo.zareasofinterest
            from zrtaddressmo
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                        )
                    )

                description = 'Address'
                report = ArtifactHtmlReport('Locations')
                report.start_artifact_report(
                    report_folder, 'RoutineD Cloud Addresses', description
                )
                report.add_script()
                data_headers = (
                    'Address Creation Date',
                    'Address Expiration Date',
                    'Country',
                    'Country Code',
                    'Postal Code',
                    'Locality',
                    'Sublocality',
                    'Throroughfare',
                    'Subthroroughfare',
                    'Subadministrative Area',
                    'Area of Interest',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'RoutineD Cloud Addresses'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'RoutineD Cloud Addresses'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No RoutineD Cloud Addresses data available')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zrtmapitemmo.zcreationdate + 978307200, 'unixepoch'),
            datetime(zrtmapitemmo.zexpirationdate + 978307200, 'unixepoch'),
            zrtmapitemmo.zname,
            zrtaddressmo.zcountry,
            zrtaddressmo.zcountrycode,
            zrtaddressmo.zpostalcode,
            zrtaddressmo.zlocality,
            zrtaddressmo.zsublocality,
            zrtaddressmo.zthoroughfare,
            zrtaddressmo.zsubthoroughfare,
            zrtaddressmo.zsubadministrativearea,
            zrtaddressmo.zareasofinterest,
            zrtmapitemmo.zlatitude,
            zrtmapitemmo.zlongitude,
            zrtmapitemmo.zuncertainty
            from zrtmapitemmo, zrtaddressmo
            where  zrtmapitemmo.z_pk == zrtaddressmo.zmapitem
            """
            )
            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                        )
                    )

                description = 'RoutineD Cloud Map Items'
                report = ArtifactHtmlReport('Locations')
                report.start_artifact_report(
                    report_folder, 'RoutineD Cloud Map Items', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Map Item Expiration Date',
                    'Map Item Name',
                    'Country',
                    'Country Code',
                    'Postal Code',
                    'Locality',
                    'Sublocality',
                    'Throroughfare',
                    'Subthroroughfare',
                    'Subadministrative Area',
                    'Area of Interest',
                    'Latitude',
                    'Longitude',
                    'Uncertainty',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'RoutineD Cloud Map Items'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'RoutineD Cloud Map Items'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'RoutineD Cloud Map Items'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)

        else:
            logfunc('No RoutineD Map Items Cloud-V2.sqlite data available')

        if version.parse(iOSversion) >= version.parse("13"):
            cursor.execute(
                """
            select
            datetime(zrtlearnedvisitmo.zentrydate + 978307200, 'unixepoch'),
            datetime(zrtlearnedvisitmo.zexitdate + 978307200, 'unixepoch'),
            (zexitdate-zentrydate)/60.00,
            zrtlearnedvisitmo.zplace,
            zrtlearnedvisitmo.zdatapointcount,
            zrtaddressmo.zcountry,
            zrtaddressmo.zcountrycode,
            zrtaddressmo.zpostalcode,
            zrtaddressmo.zlocality,
            zrtaddressmo.zsublocality,
            zrtaddressmo.zthoroughfare,
            zrtaddressmo.zsubthoroughfare,
            zrtaddressmo.zsubadministrativearea,
            zrtaddressmo.zareasofinterest,
            zrtlearnedvisitmo.zlocationuncertainty,
            zrtlearnedvisitmo.zconfidence,
            datetime(zrtlearnedvisitmo.zcreationdate + 978307200, 'unixepoch'),
            datetime(zrtlearnedvisitmo.zexpirationdate + 978307200, 'unixepoch'),
            zrtdevicemo.zdeviceclass,
            zrtdevicemo.zdevicemodel,
            zrtdevicemo.zdevicename,
            datetime(zrtlearnedplacemo.zcreationdate + 978307200, 'unixepoch'),
            datetime(zrtlearnedplacemo.zexpirationdate + 978307200, 'unixepoch'),
            datetime(zrtaddressmo.zcreationdate + 978307200, 'unixepoch'),
            zrtlearnedvisitmo.zlocationlatitude,
            zrtlearnedvisitmo.zlocationlongitude,
            datetime(zrtmapitemmo.zcreationdate + 978307200, 'unixepoch'),
            datetime(zrtmapitemmo.zexpirationdate + 978307200, 'unixepoch'),
            zrtmapitemmo.zlatitude,
            zrtmapitemmo.zlongitude,
            zrtmapitemmo.zuncertainty,
            zrtmapitemmo.zdisplaylanguage,
            zrtmapitemmo.zname
            from
            zrtlearnedvisitmo, zrtdevicemo, zrtlearnedplacemo,  zrtaddressmo, zrtmapitemmo
            where zrtlearnedvisitmo.zdevice = zrtdevicemo.z_pk
            and zrtlearnedplacemo.z_pk = zrtlearnedvisitmo.zplace
            and zrtaddressmo.zmapitem = zrtlearnedplacemo.zmapitem
            and zrtmapitemmo.z_pk = zrtlearnedplacemo.zmapitem
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                            row[15],
                            row[16],
                            row[17],
                            row[18],
                            row[19],
                            row[20],
                            row[21],
                            row[22],
                            row[23],
                            row[24],
                            row[25],
                            row[26],
                            row[27],
                            row[28],
                            row[29],
                            row[30],
                            row[31],
                            row[32],
                        )
                    )

                description = 'Significant Locations - Vist Entry & Exit (Historical)'
                report = ArtifactHtmlReport('Locations')
                report.start_artifact_report(
                    report_folder, 'RoutineD Cloud Visit Entry Exit', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Visit Exit',
                    'Visit Time (Minutes)',
                    'Place ID',
                    'Data Point Count',
                    'Country',
                    'Country Code',
                    'Postal Code',
                    'Locality',
                    'Sublocality',
                    'Throroughfare',
                    'Subthroroughfare',
                    'Subadministrative Area',
                    'Area of Interest',
                    'Location Uncertainty',
                    'Confidence',
                    'Visit Creation',
                    'Visit Expiration',
                    'Device Class',
                    'Device Model',
                    'Device Name',
                    'Learned Placed Creation',
                    'Learned Place Expiration',
                    'Address Creation',
                    'Latitude',
                    'Longitude',
                    'Map Item Creation Date',
                    'Map Item Expiration Date',
                    'Map Item Latitude',
                    'Map Item Longitude',
                    'Uncertainty',
                    'Map Item Language',
                    'Map Item Name',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'RoutineD Cloud Visit Entry Exit'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'RoutineD Cloud Visit Entry Exit'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'RoutineD Cloud Visit Entry Exit'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)

        else:
            logfunc(
                'No RoutineD Significant Locations - Vist Entry & Exit (Historical)'
            )



import plistlib

from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv

from artifacts.Artifact import Artifact


class KeyboardAppUsage(ab.Artifact):

    _name = 'Keyboard Application Usage'
    _search_dirs = '*/private/var/mobile/Library/Keyboard/app_usage_database.plist'
    _category = 'Keyboard Application Usage'

    def get(self, files_found, seeker):
        data_list = []

        for file_found in files_found:
            file_found = str(file_found)
            with open(file_found, "rb") as plist_file:
                plist_content = plistlib.load(plist_file)
                for app in plist_content:
                    for entry in plist_content[app]:
                        data_list.append(
                            (
                                entry['startDate'],
                                app,
                                entry['appTime'],
                                ', '.join(map(str, entry['keyboardTimes'])),
                            )
                        )

        if len(data_list) > 0:
            report = ArtifactHtmlReport('Keyboard Application Usage')
            report.start_artifact_report(report_folder, 'Keyboard Application Usage')
            report.add_script()
            data_headers = (
                'Date',
                'Application Name',
                'Application Time Used in Seconds',
                'Keyboard Times Used in Seconds',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Keyboard Application Usage'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Keyboard Application Usage'
            timeline(report_folder, tlactivity, data_list, data_headers)

        else:
            logfunc('No Keyboard Application Usage found')



from html_report.artifact_report import ArtifactHtmlReport
from helpers import open_sqlite_db_readonly, timeline, tsv

from artifacts.Artifact import Artifact


class QueryPredictions(ab.Artifact):
    _name = 'Query Predictions'
    _search_dirs = '**/query_predictions.db'
    _category = 'SMS & iMessage'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        select
        datetime(creationTimestamp, "UNIXEPOCH") as START,
        content,
        isSent,
        conversationId,
        id,
        uuid
        from messages
        """
        )
        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3], row[4], row[5]))

            report = ArtifactHtmlReport('Query Predictions')
            report.start_artifact_report(report_folder, 'Query Predictions')
            report.add_script()
            data_headers = (
                'Timestamp',
                'Content',
                'Is Sent?',
                'Conversation ID',
                'ID',
                'UUID',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Query Predictions'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Query Predictions'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in table')

        db.close()



import plistlib

from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv

from artifacts.Artifact import Artifact


class CacheRoutesGmap(ab.Artifact):

    _name = 'Google Maps Cache Routes'
    _search_dirs = '**/Library/Application Support/CachedRoutes/*.plist'
    _category = 'Locations'
    _web_icon = WebIcon.MAP_PIN

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        data_list = []
        for file_found in files_found:
            file_found = str(file_found)

            with open(file_found, 'rb') as f:
                deserialized = plistlib.load(f)
                length = len(deserialized['$objects'])
                for x in range(length):
                    try:
                        lat = deserialized['$objects'][x]['_coordinateLat']
                        lon = deserialized['$objects'][x][
                            '_coordinateLong'
                        ]  # lat longs
                        data_list.append((file_found, lat, lon))
                    except:  # noqa
                        pass

        if len(data_list) > 0:
            description = 'Google Maps Cache Routes'
            report = ArtifactHtmlReport('Locations')
            report.start_artifact_report(
                report_folder, 'Google Maps Cache Routes', description
            )
            report.add_script()
            data_headers = ('Source File', 'Latitude', 'Longitude')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Google Maps Cache Routes'
            tsv(report_folder, data_headers, data_list, tsvname)



from html_report.artifact_report import ArtifactHtmlReport
from packaging import version  # use to search per version number
from helpers import open_sqlite_db_readonly

import artifacts.artGlobals  # use to get iOS version -> iOSversion = artifacts.artGlobals.versionf

from artifacts.Artifact import Artifact


class TCC(ab.Artifact):

    _name = 'TCC - Permissions'
    _search_dirs = '*TCC.db*'
    _category = 'App Permissions'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)

        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select client,
            service,
            datetime(last_modified,'unixepoch')
            from access
            order by client
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2]))

            if usageentries > 0:
                report = ArtifactHtmlReport('TCC - Permissions')
                report.start_artifact_report(report_folder, 'TCC - Permissions')
                report.add_script()
                data_headers = ('Bundle ID', 'Permissions', 'Last Modified Timestamp')
                report.write_artifact_data_table(
                    data_headers, data_list, file_found, html_escape=False
                )
                report.end_artifact_report()

                """
                tsvname = 'InteractionC Contacts'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'InteractonC Contacts'
                timeline(report_folder, tlactivity, data_list, data_headers)
                """
            else:
                logfunc('No data available in TCC database.')

            db.close()



from dataclasses import dataclass

from xleapp import Artifact, WebIcon, Search, timed


@dataclass
class AggDictPasscode(Artifact):
    def __post_init__(self):
        self.name = "Aggregate Dictionary Passcode State"
        self.category = "Aggregate Dictionary"
        self.web_icon = WebIcon.BOOK

    @timed
    @Search("*/AggregateDictionary/ADDataStore.sqlitedb")
    def process(self):
        for fp in self.found:
            cursor = fp().cursor()

            cursor.execute(
                """
                select
                date(dayssince1970*86400, 'unixepoch'),
                key,
                value
                from
                scalars
                where key like 'com.apple.passcode.numpasscode%'
                """,
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2]))

                self.data = data_list



import os
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import is_platform_windows, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class KikMessages(ab.Artifact):

    _name = 'Kik Messages'
    _search_dirs = '**/kik.sqlite*'
    _category = 'Kik'

    def get(self, files_found, seeker):

        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('.sqlite'):
                break

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT
        datetime(ZKIKMESSAGE.ZRECEIVEDTIMESTAMP +978307200,'UNIXEPOCH') AS RECEIVEDTIME,
        datetime(ZKIKMESSAGE.ZTIMESTAMP +978307200,'UNIXEPOCH') as TIMESTAMP,
        ZKIKMESSAGE.ZBODY,
        case ZKIKMESSAGE.ZTYPE
            when 1 then 'rcvd'
            when 2 then 'sent'
            when 3 then 'grp admin'
            when 4 then 'grp msg'
            else 'unkn' end as 'Type',
        ZKIKMESSAGE.ZUSER,
        ZKIKUSER.ZDISPLAYNAME,
        ZKIKUSER.ZUSERNAME,
        ZKIKATTACHMENT.ZCONTENT
        from ZKIKMESSAGE
        left join ZKIKUSER on ZKIKMESSAGE.ZUSER = ZKIKUSER.Z_PK
        left join ZKIKATTACHMENT on ZKIKMESSAGE.Z_PK = ZKIKATTACHMENT.ZMESSAGE
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []

        if usageentries > 0:
            for row in all_rows:
                data_list.append(
                    (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                )

            description = 'Kik Messages'
            report = ArtifactHtmlReport('Kik Messages')
            report.start_artifact_report(report_folder, 'Kik Messages', description)
            report.add_script()
            data_headers = (
                'Received Time',
                'Timestamp',
                'Message',
                'Type',
                'User',
                'Display Name',
                'Username',
                'Content',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Kik Messages'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Kik Messages'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Kik data available')

        db.close()



import glob
import os
import pathlib
import plistlib
import sqlite3
import json

from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv, timeline, is_platform_windows, open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class SafariHistory(ab.Artifact):
    _name = 'Safari History'
    _search_dirs = '**/Safari/History.db'
    _category = 'Safari Browser'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()

        cursor.execute(
            """
        select
        datetime(history_visits.visit_time+978307200,'unixepoch'),
        history_items.url,
        history_items.visit_count,
        history_visits.title,
        case history_visits.origin
        when 1 then "icloud synced"
        when 0 then "visited local device"
        else history_visits.origin
        end,
        history_visits.load_successful,
        history_visits.id,
        history_visits.redirect_source,
        history_visits.redirect_destination
        from history_items, history_visits
        where history_items.id = history_visits.history_item
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append(
                    (
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                        row[7],
                        row[8],
                    )
                )

            description = ''
            report = ArtifactHtmlReport('Safari Browser')
            report.start_artifact_report(report_folder, 'History', description)
            report.add_script()
            data_headers = (
                'Visit Time',
                'URL',
                'Visit Count',
                'Title',
                'iCloud Sync',
                'Load Successful',
                'Visit ID',
                'Redirect Source',
                'Redirect Destination',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Safari Browser History'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Safari Browser History'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in table')

        db.close()



import base64
import gzip
import sqlite3

# import artifacts.artGlobals
import struct
import zlib

from html_report.artifact_report import ArtifactHtmlReport
from helpers import open_sqlite_db_readonly, timeline, tsv

from artifacts.Artifact import Artifact


class GeodMapTiles(ab.Artifact):

    _name = 'GeoD Map Tiles'
    _search_dirs = '**/MapTiles.sqlitedb'
    _category = 'Geolocation'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        # os.chmod(file_found, 0o0777)
        db = open_sqlite_db_readonly(file_found)
        db.row_factory = sqlite3.Row
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT datetime(access_times.timestamp, 'unixepoch') as timestamp, key_a, key_b, key_c, key_d, tileset, data, size, etag
        FROM data
        INNER JOIN access_times on data.rowid = access_times.data_pk
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                tcol_places = ''
                vmp4_places = ''
                data_parsed = ''
                data = row['data']
                if data:  # NULL sometimes
                    if (
                        len(data) >= 11
                        and data[:11] == b'\xff\xd8\xff\xe0\x00\x10\x4a\x46\x49\x46\x00'
                    ):
                        img_base64 = base64.b64encode(data).decode('utf-8')
                        img_html = f'<img src="data:image/jpeg;base64, {img_base64}" alt="Map Tile" />'
                        data_parsed = img_html
                    elif len(data) >= 4 and data[:4] == b'TCOL':
                        vmp4_places, tcol_places = parsetcol(data)
                        vmp4_places = ", ".join(vmp4_places)
                        tcol_places = ", ".join(tcol_places)
                    elif len(data) >= 4 and data[:4] == b'VMP4':
                        vmp4_places = parsevmp4(data)
                        vmp4_places = ", ".join(vmp4_places)
                # else:
                # header_bytes = data[:28]
                # hexdump = generate_hexdump(header_bytes, 5) if header_bytes else ''
                # data_parsed = hexdump

                data_list.append(
                    (
                        row['timestamp'],
                        tcol_places,
                        vmp4_places,
                        data_parsed,
                        get_hex(row['tileset']),
                        get_hex(row['key_a']),
                        get_hex(row['key_b']),
                        get_hex(row['key_c']),
                        get_hex(row['key_d']),
                    )
                )
                # row['size']) , row['etag']))
            description = ''
            report = ArtifactHtmlReport('Geolocation')
            report.start_artifact_report(report_folder, 'Map Tile Cache', description)
            report.add_script()
            data_headers = (
                "Timestamp",
                "Places_from_VLOC",
                "Labels_in_tile",
                "Image",
                "Tileset",
                "Key A",
                "Key B",
                "Key C",
                "Key D",
            )  # , "Size", "ETAG")
            report.write_artifact_data_table(
                data_headers, data_list, file_found, html_escape=False
            )
            report.end_artifact_report()

            tsvname = 'Geolocation Map Tiles'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Geolocation Map tiles'
            timeline(report_folder, tlactivity, data_list, data_headers)

        else:
            logfunc('No data available for Geolocation')

        db.close()


def readvloc(data):
    names = []
    total_len = len(data)
    pos = 8
    while pos < total_len:
        if data[pos] < 0x80:
            skip_len = 2
        else:
            skip_len = 3
        end_pos = data[pos + skip_len :].find(b'\0')
        if end_pos >= 0:
            name = data[pos + skip_len : pos + skip_len + end_pos].decode(
                'utf8', 'ignore'
            )
            if name:
                names.append(name)
            pos += skip_len + end_pos + 1
        else:
            break
    return names


def parsetcol(data):
    """returns tuple (VMP4 places, VLOC places)"""
    tcol_places = []
    data_size = len(data)
    if data_size >= 8:
        tcol_data_offset = struct.unpack('<I', data[4:8])[0]
        tcol_compressed_data = data[tcol_data_offset:]
        if tcol_compressed_data:
            try:
                tcol_places = gzip.decompress(tcol_compressed_data)
                # print("VLOC ->", tcol_places)
            except (OSError, EOFError, zlib.error) as ex:
                logfunc('Gzip decompression error from parsetcol() - ' + str(ex))
                tcol_places = ''
        vmp4_places = parsevmp4(data[8:tcol_data_offset])
        return vmp4_places, readvloc(tcol_places)


def parsevmp4(data):
    num_items = struct.unpack('<H', data[6:8])[0]
    pos = 8
    for x in range(num_items):
        item_type, offset, size = struct.unpack("<HII", data[pos : pos + 10])
        if item_type == 10:
            item_data = data[offset : offset + size]
            if item_data[0] == 1:
                compressed_data = item_data[5:]
                try:
                    places_data = zlib.decompress(compressed_data)
                except zlib.error as ex:
                    logfunc('Zlib decompression error from parsevmp4() - ' + str(ex))
                    places_data = ''
            else:
                places_data = item_data[1:]
            # print("VMP4 ->", places_data.rstrip(b'\0').split(b'\0'))
            return [
                x.decode('UTF8', 'ignore')
                for x in places_data.rstrip(b'\0').split(b'\0')
            ]
        pos += 10
    return []


def get_hex(num):
    if num:
        return hex(num).upper()
    return ''



import datetime
import os
import shutil
from html import escape
from urllib.parse import quote

from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, tsv
from helpers.parsers import ktxparser
from PIL import Image

from artifacts.Artifact import Artifact


class AppSnapShots(ab.Artifact):

    _name = 'App Snap Shots'
    _search_dirs = ('**/Library/Caches/Snapshots/*', '**/SplashBoard/Snapshots/*')
    _category = 'Installed Apps'
    _web_icon = WebIcon.PACKAGE

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):

        slash = '\\' if is_platform_windows() else '/'
        data_headers = ('App Name', 'Source Path', 'Date Modified', 'Snapshot')

        # Format=  [ [ 'App Name', 'ktx_path', mod_date, 'png_path' ], .. ]
        data_list = []

        for file_found in files_found:
            file_found = str(file_found)
            if os.path.isdir(file_found):
                continue
            if file_found.lower().endswith('.ktx'):
                # too small, they are blank
                if os.path.getsize(file_found) < 2500:
                    continue
                parts = file_found.split(slash)
                if parts[-2] != 'downscaled':
                    app_name = parts[-2].split(' ')[0]
                else:
                    app_name = parts[-3].split(' ')[0]

                png_path = os.path.join(
                    report_folder, app_name + '_' + parts[-1][:-4] + '.png'
                )
                if save_ktx_to_png_if_valid(file_found, png_path):
                    last_modified_date = datetime.datetime.fromtimestamp(
                        os.path.getmtime(file_found)
                    )
                    data_list.append(
                        [app_name, file_found, last_modified_date, png_path]
                    )

            elif file_found.lower().endswith('.jpeg'):
                parts = file_found.split(slash)
                if parts[-2] != 'downscaled':
                    app_name = parts[-2].split(' ')[0]
                else:
                    app_name = parts[-3].split(' ')[0]
                if app_name.startswith('sceneID'):
                    app_name = app_name[8:]
                # if app_name.endswith('-default'):
                #    app_name = app_name[:-8]
                dash_pos = app_name.find('-')
                if dash_pos > 0:
                    app_name = app_name[0:dash_pos]

                jpg_path = os.path.join(report_folder, app_name + '_' + parts[-1])
                if shutil.copy2(file_found, jpg_path):
                    last_modified_date = datetime.datetime.fromtimestamp(
                        os.path.getmtime(file_found)
                    )
                    data_list.append(
                        [app_name, file_found, last_modified_date, jpg_path]
                    )

        if len(data_list):
            description = "Snapshots saved by iOS for individual apps appear here. Blank screenshots are excluded here. Dates and times shown are from file modified timestamps"
            report = ArtifactHtmlReport('App Snapshots (screenshots)')
            report.start_artifact_report(report_folder, 'App Snapshots', description)
            report.add_script()
            report_folder_name = os.path.basename(report_folder.rstrip(slash))
            data_list_for_report = []
            for app_name, ktx_path, mod_date, png_path in data_list:
                dir_path, base_name = os.path.split(png_path)
                img_html = '<a href="{1}/{0}"><img src="{1}/{0}" class="img-fluid" style="max-height:300px; max-width:400px"></a>'.format(
                    quote(base_name), quote(report_folder_name)
                )
                data_list_for_report.append(
                    (escape(app_name), escape(ktx_path), mod_date, img_html)
                )
            report.write_artifact_data_table(
                data_headers,
                data_list_for_report,
                '',
                html_escape=False,
                write_location=False,
            )
            report.end_artifact_report()

            tsvname = 'App Snapshots'
            tsv_headers = ('App Name', 'Source Path', 'Date Modified')
            tsv(report_folder, tsv_headers, data_list, tsvname)
        else:
            logfunc('No snapshots available')


def save_ktx_to_png_if_valid(ktx_path, save_to_path):
    """Excludes all white or all black blank images"""

    with open(ktx_path, 'rb') as f:
        ktx = ktxparser.KTX_reader()
        try:
            if ktx.validate_header(f):
                data = ktx.get_uncompressed_texture_data(f)
                dec_img = Image.frombytes(
                    'RGBA',
                    (ktx.pixelWidth, ktx.pixelHeight),
                    data,
                    'astc',
                    (3, 4, False),
                )
                # either all black or all white
                # https://stackoverflow.com/questions/14041561/python-pil-detect-if-an-image-is-completely-black-or-white
                # if sum(dec_img.convert("L").getextrema()) in (-1, 2):
                #     logfunc('Skipping image as it is blank')
                #     return False

                dec_img.save(save_to_path, "PNG")
                return True
        except (OSError, ValueError, ktxparser.liblzfse.error) as ex:
            logfunc(f'Had an exception - {str(ex)}')
    return False



import os
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class IOSMail(ab.Artifact):

    _name = 'IOS Mail'
    _search_dirs = '**/private/var/mobile/Library/Mail/* Index*'
    _category = 'Emails'

    def get(self, files_found, seeker):
        iOSversion = artifacts.artGlobals.versionf

        if version.parse(iOSversion) <= version.parse("11"):
            logfunc("Unsupported version for iOS emails in iOS " + iOSversion)
            return ()

        if version.parse(iOSversion) < version.parse("13"):
            head, end = os.path.split(files_found[0])
            db = sqlite3.connect(os.path.join(report_folder, "emails.db"))
            cursor = db.cursor()
            cursor.execute(
                """
            create table email1(rowid int, ds text, dr text, size int, sender text, messid int, subject text, receipt text, cc text, bcc text)
            """
            )
            db.commit()

            cursor.execute(
                """
            create table email2(rowid int, data text)
            """
            )
            db.commit()
            db.close()

            db = sqlite3.connect(os.path.join(head, "Envelope Index"))
            db.execute(f'ATTACH DATABASE "{head}/Protected Index" AS PI')
            db.execute(f'ATTACH DATABASE "{report_folder}/emails.db" AS emails')

            cursor = db.cursor()
            cursor.execute(
                """
            select
            main.messages.ROWID,
            main.messages.date_sent,
            main.messages.date_received,
            main.messages.size,
            PI.messages.sender,
            PI.messages.message_id,
            PI.messages.subject,
            PI.messages._to,
            PI.messages.cc,
            PI.messages.bcc
            from main.messages, PI.messages
            where main.messages.ROWID =  PI.messages.message_id
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                print(f"Total emails {usageentries}")
                for row in all_rows:
                    # print(row)
                    datainsert = (
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                        row[7],
                        row[8],
                        row[9],
                    )
                    cursor.execute(
                        "INSERT INTO emails.email1 (rowid, ds, dr, size, sender, messid, subject, receipt, cc, bcc) VALUES (?,?,?,?,?,?,?,?,?,?)",
                        datainsert,
                    )
                    db.commit()
            else:
                print("Zero rows")

            cursor = db.cursor()
            cursor.execute(
                """
            select
            main.messages.ROWID,
            PI.message_data.data
            from main.message_data, main.messages, PI.messages, PI.message_data
            where main.messages.ROWID = main.message_data.message_id and PI.messages.message_id = main.message_data.message_id
            and PI.message_data.message_data_id = main.message_data.ROWID
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                print(f"Total emails with message data {usageentries}")
                for row in all_rows:
                    datainsert = (
                        row[0],
                        row[1],
                    )
                    cursor.execute(
                        "INSERT INTO emails.email2 (rowid, data)  VALUES(?,?)",
                        datainsert,
                    )
                    db.commit()
            else:
                print("Zero rows")

            cursor.execute(
                """
            select
            email1.rowid,
            datetime(email1.ds, 'unixepoch') as ds,
            datetime(email1.dr, 'unixepoch') as dr,
            email1.sender,
            email1.messid,
            email1.subject,
            email1.receipt,
            email1.cc,
            email1.bcc,
            email2.data
            from email1
            left outer join email2
            on email2.rowid = email1.rowid
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[9],
                            row[7],
                            row[8],
                            row[0],
                        )
                    )

                file_found = head
                description = ''
                report = ArtifactHtmlReport('iOS Mail')
                report.start_artifact_report(report_folder, 'Emails', description)
                report.add_script()
                data_headers = (
                    'Date Sent',
                    'Date Received',
                    'Sender',
                    'Message ID',
                    'Subject',
                    'Recipient',
                    'Message',
                    'CC',
                    'BCC',
                    'Row ID',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'iOS Mail'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'iOS Mail'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc("No iOS emails available")
            db.close()

        if version.parse(iOSversion) >= version.parse("13"):
            head, end = os.path.split(files_found[0])
            db = sqlite3.connect(os.path.join(head, "Envelope Index"))
            db.execute(f'ATTACH DATABASE "{head}/Protected Index" AS PI')

            cursor = db.cursor()
            cursor.execute(
                """
            SELECT
            datetime(main.messages.date_sent, 'UNIXEPOCH') as datesent,
            datetime(main.messages.date_received, 'UNIXEPOCH') as datereceived,
            PI.addresses.address,
            PI.addresses.comment,
            PI.subjects.subject,
            PI.summaries.summary,
            main.messages.read,
            main.messages.flagged,
            main.messages.deleted,
            main.mailboxes.url
            from main.mailboxes, main.messages, PI.subjects, PI.addresses, PI.summaries
            where main.messages.subject = PI.subjects.ROWID
            and main.messages.sender = PI.addresses.ROWID
            and main.messages.summary = PI.summaries.ROWID
            and main.mailboxes.ROWID = main.messages.mailbox
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                        )
                    )

                file_found = head
                description = ''
                report = ArtifactHtmlReport('iOS Mail')
                report.start_artifact_report(report_folder, 'Emails', description)
                report.add_script()
                data_headers = (
                    'Date Sent',
                    'Date Received',
                    'Address',
                    'Comment',
                    'Subject',
                    'Summary',
                    'Read?',
                    'Flagged?',
                    'Deleted',
                    'Mailbox',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'iOS Mail'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'iOS Mail'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc("No iOS emails available")



import datetime

from helpers.db import open_sqlite_db_readonly
from helpers import timeline, tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class FilesAppsClient(ab.Artifact):

    _name = 'Files App - iCloud Client Items'
    _search_dirs = '*private/var/mobile/Library/Application Support/CloudDocs/session/db/client.db*'
    _category = 'Files App'
    _web_icon = WebIcon.FILES_TEXT

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('client.db'):
                break

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT
        item_birthtime,
        item_filename,
        version_mtime
        FROM
        client_items
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                birthtime = datetime.datetime.fromtimestamp(row[0])
                versionmtime = datetime.datetime.fromtimestamp(row[2])
                data_list.append((birthtime, row[1], versionmtime))

            description = '	Items stored in iCloud Drive with metadata about files. '
            report = ArtifactHtmlReport('Files App - iCloud Client Items')
            report.start_artifact_report(
                report_folder, 'Files App - iCloud Client Items', description
            )
            report.add_script()
            data_headers = ('Birthtime', 'Filename', 'Version Modified Time')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Files App - iCloud Client Items'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Files App - iCloud Client Items'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Files App - iCloud Client Items data available')



from helpers import timeline, tsv
from helpers.db import open_sqlite_db_readonly
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class AppleWalletTransactions(ab.Artifact):
    _name = 'Apple Wallet Transaction'
    _search_dirs = '**/passes23.sqlite'
    _category = 'Apple Wallet'
    _web_icon = WebIcon.DOLLAR_SIGN

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        select
        datetime(payment_transaction.transaction_date + 978307200,'unixepoch') AS "Transaction Date",
        payment_transaction.merchant_name AS "Merchant",
        payment_transaction.currency_code AS "Currency Type",
        CAST (payment_transaction.amount as REAL)/100  AS  "Currency Amount",
        payment_transaction.peer_payment_counterpart_handle AS "Peer Payment Handle",
        payment_transaction.peer_payment_memo AS "Payment Memo",
        payment_transaction.transaction_status AS "Transaction Status",
        payment_transaction.transaction_type AS "Transaction Type"
        from payment_transaction
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                data_list.append(
                    (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                )

            report = ArtifactHtmlReport('Apple Wallet - Transactions')
            report.start_artifact_report(report_folder, 'Apple Wallet - Transactions')
            report.add_script()
            data_headers = (
                'Transaction Date',
                'Merchant',
                'Currency Type',
                'Currency Amount',
                'Peer Payment Handle',
                'Payment Memo',
                'Transaction Status',
                'Transaction Type',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Apple Wallet - Transactions'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Apple Wallet - Transactions'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Apple Wallet transactions available')

        db.close()



import datetime
import plistlib
from dataclasses import dataclass

from xleapp import Artifact, Search, core_artifact, timed


@core_artifact
@dataclass
class LastBuild(Artifact):
    def __post_init__(self):
        self.name = "Last Build"
        self.category = "IOS Build"

    @timed
    @Search("*LastBuildInfo.plist")
    def process(self):
        data_list = []
        device_info = self.app.device
        for fp in self.found:
            pl = plistlib.load(fp())
            for key, value in pl.items():
                data_list.append((key, value))
                if key in ["ProductVersion", "ProductBuildVersion", "ProductName"]:
                    device_info.update({key: value})
            self.data = data_list


@core_artifact
@dataclass
class ITunesBackupInfo(Artifact):
    def __post_init__(self):
        self.name = "iTunesBackup"
        self.category = "IOS Build"

    @timed
    @Search("*LastBuildInfo.plist")
    def process(self):
        data_list = []
        device_info = self.app.device
        fp = self.found
        pl = plistlib.load(fp())
        for key, value in pl.items():
            if (
                isinstance(value, str)
                or isinstance(value, int)
                or isinstance(value, datetime.datetime)
            ):

                data_list.append((key, value))
                if key in (
                    "Build Version",
                    "Device Name",
                    "ICCID",
                    "IMEI",
                    "Last Backup Date",
                    "MEID",
                    "Phone Number",
                    "Product Name",
                    "Product Type",
                    "Product Version",
                    "Serial Number",
                ):
                    device_info.update({key: value})

            elif key == "Installed Applications":
                data_list.append((key, ", ".join(value)))

        self.data = data_list



from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv

import artifacts.artGlobals  # use to get iOS version -> iOSversion = artifacts.artGlobals.versionf

from artifacts.Artifact import Artifact


class DHCPHotspotClients(ab.Artifact):

    _name = 'DHCP Hotspot Clients'
    _search_dirs = '**/private/var/db/dhcpd_leases*'
    _category = 'DHCP'
    _web_icon = WebIcon.SETTINGS

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        data_list = []
        reportval = ''
        with open(file_found, "r") as filefrom:
            for line in filefrom:
                cline = line.strip()
                if cline == "{":
                    reportval = reportval + (
                        "<table><tr><td>Key</td><td>Values</td></tr>"
                    )
                elif cline == "}":
                    reportval = reportval + ("</table>")
                    data_list.append((reportval,))
                    reportval = ''
                # elif cline == '':
                # 	f.write('<br>')
                else:
                    ll = cline.split("=")
                    reportval = reportval + (f"<tr><td>{ll[0]}</td>")
                    reportval = reportval + (f"<td>{ll[1]}</td></tr>")

        if len(data_list) > 0:
            report = ArtifactHtmlReport('DHCP Hotspot Clients')
            report.start_artifact_report(report_folder, 'Hotspot Clients')
            report.add_script()
            data_headers = ('Hotspot Clients',)
            report.write_artifact_data_table(
                data_headers, data_list, file_found, html_escape=False
            )
            report.end_artifact_report()

            tsvname = 'DHCP Hotspot Clients'
            tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data available')



import glob
import os
import nska_deserialize as nd
import sqlite3
import datetime

from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv, timeline, is_platform_windows, open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class FilesAppsSM(ab.Artifact):

    _name = 'File Apps - Filenames'
    _search_dirs = '*private/var/mobile/Containers/Shared/AppGroup/*/smartfolders.db*'
    _category = 'Files App'

    def get(self, files_found, seeker):
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('smartfolders.db'):
                break

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT *
        FROM
        FILENAMES
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:

            for row in all_rows:

                output_file = open(
                    os.path.join(report_folder, row[2] + '.bplist'), "wb"
                )
                output_file.write(row[1])
                output_file.close()

                with open(os.path.join(report_folder, row[2] + '.bplist'), "rb") as f:
                    deserialized_plist = nd.deserialize_plist(f)
                for x, y in deserialized_plist.items():
                    if x == '_creationDate':
                        creationdate = y
                    if x == '_contentModificationDate':
                        contentmodificationdate = y
                    if x == '_flags':
                        flags = y
                    if x == '_userInfo':
                        userinfo = y
                    if x == '_childItemCount':
                        childitemcount = y
                lasthitdate = datetime.datetime.fromtimestamp(row[3])

                data_list.append(
                    (
                        lasthitdate,
                        row[0],
                        row[2],
                        row[4],
                        creationdate,
                        contentmodificationdate,
                        userinfo,
                        childitemcount,
                        flags,
                    )
                )

                description = 'Files App - Files stored in the "On my iPad" area.'
                report = ArtifactHtmlReport('Files App - Filenames')
                report.start_artifact_report(
                    report_folder, 'Files App - Filenames', description
                )
                report.add_script()
                data_headers = (
                    'Last Hit Date',
                    'Folder ID',
                    'Filename',
                    'Frequency at Las Hit Date',
                    'Creation Date',
                    'Modification Date',
                    'User Info',
                    'Child Item Count',
                    'Flags',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Files App - Filenames'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Files App - Filenames'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Files App - Filenames data available')

        db.close()



import os
import plistlib
from pathlib import Path

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, tsv

from artifacts.Artifact import Artifact


class ICloudSharedAlbums(ab.Artifact):

    _name = 'iCloud Shared Owner Info'
    _search_dirs = '*/private/var/mobile/Media/PhotoData/PhotoCloudSharingData/*'
    _category = 'iCloud Shared Albums'

    def get(self, files_found, seeker):
        data_list_sharedemails = []
        data_list_sharedpersoninfos = []
        data_list_sharedinfos = []
        data_list_cloudinfo = []

        for file_found in files_found:
            file_found = str(file_found)
            pathedhead, pathedtail = os.path.split(file_found)

            if os.path.isfile(file_found):
                if pathedtail == 'cloudSharedEmails.plist':
                    with open(file_found, "rb") as fp:
                        pl = plistlib.load(fp)
                        for x, y in pl.items():
                            data_list_sharedemails.append((x, y))

                elif pathedtail == 'cloudSharedPersonInfos.plist':
                    with open(file_found, "rb") as fp:
                        pl = plistlib.load(fp)
                        for x, y in pl.items():

                            email = ''
                            firstname = ''
                            lastname = ''
                            fullname = ''

                            for a, b in y.items():
                                if a == 'email':
                                    email = b
                                if a == 'firstName':
                                    firstname = b
                                if a == 'fullName':
                                    fullname = b
                                if a == 'emails':
                                    email = b[0]
                                if a == 'lastName':
                                    lastname = b

                            file_found_persons = file_found
                            data_list_sharedpersoninfos.append(
                                (email, firstname, lastname, fullname, x)
                            )

                elif pathedtail == 'Info.plist':
                    albumid = os.path.basename(os.path.dirname(file_found))
                    albumpath = file_found
                    # print(f'Info.plist album ID: {albumid}')

                    cloudOwnerEmail = ''

                    with open(file_found, "rb") as fp:
                        pl = plistlib.load(fp)

                        cloudOwnerEmail = ''
                        cloudownerfirstname = ''
                        cloudownerlastname = ''
                        cloudpublicurlenabled = ''
                        cloudsubscriptiondate = ''
                        cloudrelationshipstate = ''
                        cloudownerhashedpersonid = ''
                        clowdoenweremail = ''
                        title = ''

                        for x, y in pl.items():
                            # print(f'{x}: {y}')
                            if x == 'cloudOwnerEmail':
                                clowdoenweremail = y
                            if x == 'cloudOwnerFirstName':
                                cloudownerfirstname = y
                            if x == 'cloudOwnerLastName':
                                cloudownerlastname = y
                            if x == 'cloudPublicURLEnabled':
                                cloudpublicurlenabled = y
                            if x == 'cloudSubscriptionDate':
                                cloudsubscriptiondate = y
                            if x == 'cloudRelationshipState':
                                cloudrelationshipstate = y
                            if x == 'title':
                                albumtitle = y
                            if x == 'cloudOwnerHashedPersonID':
                                cloudownerhashedpersonid = y

                    data_list_sharedinfos.append(
                        (
                            albumtitle,
                            albumid,
                            clowdoenweremail,
                            cloudownerfirstname,
                            cloudownerlastname,
                            cloudpublicurlenabled,
                            cloudsubscriptiondate,
                            cloudrelationshipstate,
                            cloudownerhashedpersonid,
                            albumpath,
                        )
                    )

                elif pathedtail == 'DCIM_CLOUD.plist':
                    albumid = os.path.basename(os.path.dirname(file_found))
                    albumpath = file_found
                    # print(f'Info.plist album ID: {albumid}')
                    with open(file_found, "rb") as fp:
                        pl = plistlib.load(fp)
                        for x, y in pl.items():
                            if x == 'DCIMLastDirectoryNumber':
                                dcimlastdictnum = y
                            if x == 'DCIMLastFileNumber':
                                dcimlastfilenum = y
                        data_list_cloudinfo.append(
                            (albumid, dcimlastdictnum, dcimlastfilenum, albumpath)
                        )

            else:
                pass

        if len(data_list_sharedinfos) > 0:
            location = 'See report entry'
            report = ArtifactHtmlReport('iCloud Shared Owner Info')
            report.start_artifact_report(report_folder, 'iCloud Shared Owner Info')
            report.add_script()
            data_headers = (
                'Album Title',
                'Album ID',
                'Clowd Owner Email',
                'Cloud Owner First Name',
                'Clowd Owner Lastname',
                'Cloud Public URL Enabled?',
                'Cloud Subscription Date',
                'Cloud Relationship State',
                'Cloud Ownewr Hashed Person ID',
                'File location',
            )
            report.write_artifact_data_table(
                data_headers, data_list_sharedinfos, location
            )
            report.end_artifact_report()

            tsvname = 'iCloud Shared Owner Info'
            tsv(report_folder, data_headers, data_list_sharedinfos, tsvname)
        else:
            logfunc('No iCloud Shared Owner Info')

        if len(data_list_cloudinfo) > 0:
            location = 'See report entry'
            report = ArtifactHtmlReport('iCloud Shared Album data')
            report.start_artifact_report(report_folder, 'iCloud Shared Album Data')
            report.add_script()
            data_headers = (
                'Album Name',
                'DCIM Last Directory Number',
                'DCIM LAst File Number',
                'File location',
            )
            report.write_artifact_data_table(
                data_headers, data_list_cloudinfo, location
            )
            report.end_artifact_report()

            tsvname = 'iCloud Shared Album Data'
            tsv(report_folder, data_headers, data_list_cloudinfo, tsvname)
        else:
            logfunc('No iCloud Shared Album Data')

        if len(data_list_sharedpersoninfos) > 0:
            location = file_found_persons
            report = ArtifactHtmlReport('iCloud Shared Album Persons Info')
            report.start_artifact_report(report_folder, 'iCloud Shared Person Info')
            report.add_script()
            data_headers = (
                'Email',
                'Firstname',
                'Lastname',
                'Fullname',
                'Identification',
            )
            report.write_artifact_data_table(
                data_headers, data_list_sharedpersoninfos, location
            )
            report.end_artifact_report()

            tsvname = 'iCloud Shared Person Info'
            tsv(report_folder, data_headers, data_list_sharedpersoninfos, tsvname)
        else:
            logfunc('No iCloud Shared Album Persons Info')

        if len(data_list_sharedemails) > 0:
            location = file_found
            report = ArtifactHtmlReport('iCloud Shared Albums Emails')
            report.start_artifact_report(report_folder, 'iCloud Shared Emails')
            report.add_script()
            data_headers = ('Key', 'Value')
            report.write_artifact_data_table(
                data_headers, data_list_sharedemails, location
            )
            report.end_artifact_report()

            tsvname = 'iCloud Shared Albums Emails'
            tsv(report_folder, data_headers, data_list_sharedemails, tsvname)
        else:
            logfunc('No iCloud Shared Emails')



from helpers.db import open_sqlite_db_readonly
from helpers import timeline, tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class CallHistory(ab.Artifact):

    _name = 'Call Logs'
    _search_dirs = '**/CallHistory.storedata'
    _category = 'Call Logs'
    _web_icon = WebIcon.PHONE

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT
        Z_PK as "CALL ID",
        ZADDRESS AS "ADDRESS",
        ZANSWERED AS "WAS ANSWERED",
        ZCALLTYPE AS "CALL TYPE",
        ZORIGINATED AS "ORIGINATED",
        ZDURATION AS "DURATION (IN SECONDS)",
        ZISO_COUNTRY_CODE as "ISO COUNTY CODE",
        ZLOCATION AS "LOCATION",
        ZSERVICE_PROVIDER AS "SERVICE PROVIDER",
        DATETIME(ZDATE+978307200,'UNIXEPOCH') AS "TIMESTAMP"
        FROM ZCALLRECORD
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                an = str(row[1])
                an = an.replace("b'", "")
                an = an.replace("'", "")
                data_list.append(
                    (
                        row[9],
                        row[0],
                        an,
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                        row[7],
                        row[8],
                    )
                )

            report = ArtifactHtmlReport('Call Logs')
            report.start_artifact_report(report_folder, 'Call Logs')
            report.add_script()
            data_headers = (
                'Timestamp',
                'Call ID',
                'Address',
                'Was Answered',
                'Call Type',
                'Originated',
                'Duration in Secs',
                'ISO County Code',
                'Location',
                'Service Provider',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Call Logs'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Call Logs'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Call Logs data available')

        db.close()



import os
import re
import string
from html import escape
from pathlib import Path

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, logfunc

from artifacts.Artifact import Artifact

control_chars = ''.join(map(chr, range(0, 32))) + ''.join(map(chr, range(127, 160)))
not_control_char_re = re.compile(f'[^{control_chars}]' + '{4,}')
# If  we only want ascii, use 'ascii_chars_re' below
printable_chars_for_re = (
    string.printable.replace('\\', '\\\\').replace('[', '\\[').replace(']', '\\]')
)
ascii_chars_re = re.compile(f'[{printable_chars_for_re}]' + '{4,}')


class WalStrings(ab.Artifact):
    _name = 'Strings - SQLite Journal & WAL'
    _search_dirs = ('**/*-wal', '**/*-journal')
    _category = 'SQLite Journaling'

    def get(self, files_found, seeker):
        x = 1
        data_list = []
        for file_found in files_found:
            filesize = Path(file_found).stat().st_size
            if filesize == 0:
                continue

            journalName = os.path.basename(file_found)
            outputpath = os.path.join(
                report_folder, str(x) + '_' + journalName + '.txt'
            )  # name of file in txt

            level2, level1 = os.path.split(outputpath)
            level2 = os.path.split(level2)[1]
            final = level2 + '/' + level1

            unique_items = set()  # For deduplication of strings found
            with open(outputpath, 'w') as g:
                with open(file_found, errors="ignore") as f:  # Python 3.x
                    data = f.read()
                    # for match in not_control_char_re.finditer(data): # This gets all unicode chars, can include lot of garbage if you only care about English, will miss out other languages
                    for match in ascii_chars_re.finditer(
                        data
                    ):  # Matches ONLY Ascii (old behavior) , good if you only care about English
                        if match.group() not in unique_items:
                            g.write(match.group())
                            g.write('\n')
                            unique_items.add(match.group())
                g.close()

            if unique_items:
                out = f'<a href="{final}" style = "color:blue" target="_blank">{journalName}</a>'
                data_list.append((out, file_found))
            else:
                try:
                    os.remove(outputpath)  # delete empty file
                except OSError:
                    pass
            x = x + 1

        location = ''
        description = 'ASCII strings extracted from SQLite journal and WAL files.'
        report = ArtifactHtmlReport('Strings - SQLite Journal & WAL')
        report.start_artifact_report(
            report_folder, 'Strings - SQLite Journal & WAL', description
        )
        report.add_script()
        data_headers = ('Report', 'Location')
        report.write_artifact_data_table(
            data_headers, data_list, location, html_escape=False
        )
        report.end_artifact_report()



import string
from os.path import dirname

from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv

from artifacts.Artifact import Artifact


class KeyboardLexicon(ab.Artifact):

    _name = 'Keyboard Dynamic Lexicon'
    _search_dirs = (
        '*/private/var/mobile/Library/Keyboard/*-dynamic.lm/dynamic-lexicon.dat'
    )
    _category = 'Keyboard Dynamic Lexicon'

    def get(self, files_found, seeker):
        data_list = []
        tsv_data_list = []
        for file_found in files_found:
            strings_list = []
            with open(file_found, 'rb') as dat_file:
                dat_content = dat_file.read()
                dat_content_decoded = str(dat_content, 'utf-8', 'ignore')
                found_str = ''
                for char in dat_content_decoded:
                    if char in string.printable:
                        found_str += char
                    else:
                        if found_str:
                            if len(found_str) > 3:  # reduce noise
                                strings_list.append(found_str)
                            found_str = ''

            if file_found.find("Keyboard/") >= 0:
                slash = '/'
            else:
                slash = '\\'
            location_file_found = file_found.split(f"Keyboard{slash}", 1)[1]
            data_list.append(('<br>'.join(strings_list), location_file_found))
            tsv_data_list.append((','.join(strings_list), location_file_found))

            dir_file_found = dirname(file_found).split('Keyboard', 1)[0] + 'Keyboard'

        if data_list:
            report = ArtifactHtmlReport('Keyboard Dynamic Lexicon')
            report.start_artifact_report(report_folder, 'Keyboard Dynamic Lexicon')
            report.add_script()
            data_headers = ('Found Strings', 'File Location')
            report.write_artifact_data_table(
                data_headers,
                data_list,
                dir_file_found,
                html_no_escape=['Found Strings'],
            )
            report.end_artifact_report()

            tsvname = 'Keyboard Dynamic Lexicon'
            tsv(report_folder, data_headers, tsv_data_list, tsvname)

            tlactivity = 'Keyboard Dynamic Lexicon'
            timeline(report_folder, tlactivity, tsv_data_list, data_headers)

        else:
            logfunc('No Keyboard Dynamic Lexicon data found')



import gzip
import os
import re
import shutil
import sqlite3
from pathlib import Path

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import is_platform_windows, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class PowerLogGZ(ab.Artifact):
    _name = 'Power Log GZ'
    _search_dirs = '**/Library/BatteryLife/Archives/powerlog_*.PLSQL.gz'
    _category = "Power Log Backups"

    def get(self, files_found, seeker):
        iOSversion = artifacts.artGlobals.versionf
        data_list1 = []
        data_list2 = []
        data_list3 = []
        data_list4 = []
        data_list5 = []
        data_list6 = []
        data_list7 = []
        data_list8 = []
        data_list9 = []
        data_list10 = []
        data_list11 = []
        data_list12 = []
        data_list13 = []
        data_list14 = []
        data_list15 = []
        data_list16 = []
        data_list17 = []
        data_list18 = []

        logfunc('Unzipped Powerlog databases:')
        for file_found in files_found:
            file_found = str(file_found)
            filename = Path(file_found)
            ungzipedfile = Path(filename.parent, filename.stem)
            with gzip.open(file_found, 'rb') as f_in:
                with open(ungzipedfile, 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)
                    f_out.close()
                    logfunc(str(filename.stem))
                    file_found = str(Path(filename.parent))

                    db = open_sqlite_db_readonly(str(ungzipedfile))
                    cursor = db.cursor()

                if version.parse(iOSversion) >= version.parse("9"):
                    cursor.execute(
                        """
                    select
                    datetime(timestamp, 'unixepoch'),
                    datetime(timestamplogged, 'unixepoch'),
                    applicationname,
                    assertionid,
                    assertionname,
                    audioroute,
                    mirroringstate,
                    operation,
                    pid
                    from
                    plaudioagent_eventpoint_audioapp
                    """
                    )

                    all_rows = cursor.fetchall()
                    usageentries = len(all_rows)
                    if usageentries > 0:
                        for row in all_rows:
                            data_list1.append(
                                (
                                    row[0],
                                    row[1],
                                    row[2],
                                    row[3],
                                    row[4],
                                    row[5],
                                    row[6],
                                    row[7],
                                    row[8],
                                )
                            )
                    else:
                        logfunc('No data available in Powerlog Audio Routing via app')

                if version.parse(iOSversion) >= version.parse("10"):
                    cursor.execute(
                        """
                    select
                    datetime(timestamp, 'unixepoch'),
                    bulletinbundleid,
                    timeinterval / 60,
                    count,
                    posttype
                    from
                    plspringboardagent_aggregate_sbbulletins_aggregate
                    """
                    )
                    all_rows = cursor.fetchall()
                    usageentries = len(all_rows)
                    if usageentries > 0:
                        for row in all_rows:
                            data_list2.append((row[0], row[1], row[2], row[3], row[4]))
                    else:
                        logfunc('No data available in Aggregate Bulletins')

                if version.parse(iOSversion) >= version.parse("10"):
                    cursor.execute(
                        """
                        select
                        datetime(timestamp, 'unixepoch'),
                        notificationbundleid,
                        count as "count",
                        notificationtype
                        from
                        plspringboardagent_aggregate_sbnotifications_aggregate
                        """
                    )
                    all_rows = cursor.fetchall()
                    usageentries = len(all_rows)
                    if usageentries > 0:

                        for row in all_rows:
                            data_list3.append((row[0], row[1], row[2], row[3]))
                    else:
                        logfunc('No data available in Aggregate Notifications')

            if version.parse(iOSversion) >= version.parse("9"):
                cursor.execute(
                    """
                select
                datetime(timestamp, 'unixepoch'),
                appname,
                appexecutable,
                appbundleid,
                appbuildversion,
                appbundleversion,
                apptype,
                case appdeleteddate
                when 0 then "not deleted"
                else datetime(appdeleteddate, 'unixepoch')
                end
                from
                plapplicationagent_eventnone_allapps
                """
                )

                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                if usageentries > 0:
                    for row in all_rows:
                        data_list5.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                            )
                        )
                else:
                    logfunc('No data available in Powerlog App Info')

            if version.parse(iOSversion) >= version.parse("11"):
                cursor.execute(
                    """
                select
                datetime(timestamp, 'unixepoch'),
                datetime(start, 'unixepoch'),
                datetime(end, 'unixepoch'),
                state,
                finished,
                haserror
                from
                plxpcagent_eventpoint_mobilebackupevents
                """
                )

                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                if usageentries > 0:
                    for row in all_rows:
                        data_list6.append(
                            (row[0], row[1], row[2], row[3], row[4], row[5])
                        )
                else:
                    logfunc('No data available in Powerlog Backup Info')

            if version.parse(iOSversion) >= version.parse("11"):
                cursor.execute(
                    """
                select
                datetime(appdeleteddate, 'unixepoch'),
                datetime(timestamp, 'unixepoch'),
                appname,
                appexecutable,
                appbundleid,
                appbuildversion,
                appbundleversion,
                apptype
                from
                plapplicationagent_eventnone_allapps
                where
                appdeleteddate > 0			"""
                )
                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                if usageentries > 0:
                    for row in all_rows:
                        data_list7.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                            )
                        )

            if version.parse(iOSversion) == version.parse("10"):
                cursor.execute(
                    """
                select
                datetime(appdeleteddate, 'unixepoch'),
                datetime(timestamp, 'unixepoch'),
                appname,
                appexecutable,
                appbundleid,
                appbuildversion,
                appbundleversion
                from
                plapplicationagent_eventnone_allapps
                where
                appdeleteddate > 0
                """
                )
                iall_rows = cursor.fetchall()
                usageentries = len(all_rows)
                if usageentries > 0:
                    for row in all_rows:
                        data_list7.append(
                            (row[0], row[1], row[2], row[3], row[4], row[5], row[6])
                        )

            if version.parse(iOSversion) == version.parse("9"):
                cursor.execute(
                    """
                select
                datetime(appdeleteddate, 'unixepoch'),
                datetime(timestamp, 'unixepoch'),
                appname,
                appbundleid
                from
                plapplicationagent_eventnone_allapps
                where
                appdeleteddate > 0
                """
                )
                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                if usageentries > 0:
                    for row in all_rows:
                        data_list7.append((row[0], row[1], row[2], row[3]))

            if version.parse(iOSversion) >= version.parse("10"):
                cursor.execute(
                    """
                select
                datetime(timestamp, 'unixepoch'),
                build,
                device,
                hwmodel,
                pairingid
                from
                plconfigagent_eventnone_paireddeviceconfig
                """
                )
            else:
                cursor.execute(
                    """
                select
                datetime(timestamp, 'unixepoch'),
                build,
                device
                from
                plconfigagent_eventnone_paireddeviceconfig
                """
                )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                if version.parse(iOSversion) >= version.parse("10"):
                    for row in all_rows:
                        data_list11.append((row[0], row[1], row[2], row[3], row[4]))
                else:
                    for row in all_rows:
                        data_list11.append((row[0], row[1], row[2]))

            if version.parse(iOSversion) >= version.parse("9"):
                cursor.execute(
                    """
                select
                datetime(tts + system, 'unixepoch'),
                bundleid,
                case level
                when "0" then "off"
                when "1" then "on"
                end as status,
                datetime(tts, 'unixepoch'),
                datetime(tot, 'unixepoch'),
                system
                from
                (
                select
                bundleid,
                torchid,
                tts,
                tot,
                max(toid),
                system,
                level
                from
                (
                select
                plcameraagent_eventforward_torch.timestamp as tts,
                plcameraagent_eventforward_torch.bundleid,
                plcameraagent_eventforward_torch.level,
                plcameraagent_eventforward_torch.id as "torchid",
                plstorageoperator_eventforward_timeoffset.timestamp as tot,
                plstorageoperator_eventforward_timeoffset.id as toid,
                plstorageoperator_eventforward_timeoffset.system,
                bundleid
                from
                plcameraagent_eventforward_torch
                left join
                plstorageoperator_eventforward_timeoffset
                )
                as torchest
                group by
                torchid
                )
                """
                )
                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                if usageentries > 0:
                    for row in all_rows:
                        data_list15.append(
                            (row[0], row[1], row[2], row[3], row[4], row[5])
                        )

            if version.parse(iOSversion) >= version.parse("9"):
                cursor.execute(
                    """
                select
                datetime(wifipropts + system, 'unixepoch') ,
                currentssid,
                currentchannel,
                datetime(tot, 'unixepoch') ,
                system as time_offset
                from
                (
                select
                wifiorotsid,
                wifipropts,
                tot,
                max(toi),
                currentssid,
                currentchannel,
                system
                from
                (
                select
                plwifiagent_eventbackward_cumulativeproperties.timestamp as wifipropts,
                currentssid,
                currentchannel,
                plwifiagent_eventbackward_cumulativeproperties.id as "wifiorotsid" ,
                plstorageoperator_eventforward_timeoffset.timestamp as tot,
                plstorageoperator_eventforward_timeoffset.id as toi,
                plstorageoperator_eventforward_timeoffset.system
                from
                plwifiagent_eventbackward_cumulativeproperties
                left join
                plstorageoperator_eventforward_timeoffset
                )
                as wifipropst
                group by
                wifiorotsid
                )
                """
                )
                all_rows = cursor.fetchall()
                usageentries = len(all_rows)
                if usageentries > 0:
                    for row in all_rows:
                        data_list18.append((row[0], row[1], row[2], row[3], row[4]))

        if version.parse(iOSversion) >= version.parse("9"):
            if len(data_list1) > 0:
                report = ArtifactHtmlReport('Powerlog Audio Routing via App')
                report.start_artifact_report(report_folder, 'Audio Routing')
                report.add_script()
                data_headers1 = (
                    'Timestamp',
                    'Timestamped Logged',
                    'Bundle ID',
                    'Assertion Name',
                    'Audio Route',
                    'Mirroring State',
                    'Operation',
                    'PID',
                    'Audio App Table ID',
                )
                report.write_artifact_data_table(data_headers1, data_list1, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Audio Routing via App from GZ backup'
                tsv(report_folder, data_headers1, data_list1, tsvname)

                tlactivity = 'Powerlog Audio Routing via App from GZ backup'
                timeline(report_folder, tlactivity, data_list1, data_headers1)

        if version.parse(iOSversion) >= version.parse("10"):
            if len(data_list2) > 0:
                report = ArtifactHtmlReport('Powerlog Aggregate Bulletins')
                report.start_artifact_report(report_folder, 'Aggregate Bulletins')
                report.add_script()
                data_headers2 = (
                    'Timestamp',
                    'Bulletin Bundle ID',
                    'Time Interval in Seconds',
                    'Count',
                    'Post Type',
                )
                report.write_artifact_data_table(data_headers2, data_list2, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Agg Bulletins from GZ backup'
                tsv(report_folder, data_headers2, data_list2, tsvname)

                tlactivity = 'Powerlog Agg Bulletins from GZ backup'
                timeline(report_folder, tlactivity, data_list2, data_headers2)

        if version.parse(iOSversion) >= version.parse("10"):
            if len(data_list3) > 0:
                report = ArtifactHtmlReport('Powerlog Aggregate Notifications')
                report.start_artifact_report(report_folder, 'Aggregate Notifications')
                report.add_script()
                data_headers3 = (
                    'Timestamp',
                    'Notification Bundle ID',
                    'Count',
                    'Notification Type',
                )
                report.write_artifact_data_table(data_headers3, data_list3, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Agg Notifications from GZ backup'
                tsv(report_folder, data_headers3, data_list3, tsvname)

                tlactivity = 'Powerlog Agg Notifications from GZ backup'
                timeline(report_folder, tlactivity, data_list3, data_headers3)

        if version.parse(iOSversion) >= version.parse("9"):
            if len(data_list5) > 0:
                report = ArtifactHtmlReport('Powerlog App Info')
                report.start_artifact_report(report_folder, 'App Info')
                report.add_script()
                data_headers5 = (
                    'Timestamp',
                    'App Name',
                    'App Executable Name',
                    'Bundle ID',
                    'App Build Version',
                    'App Bundle Version',
                    'App TYpe',
                    'App Deleted Date',
                )
                report.write_artifact_data_table(data_headers5, data_list5, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog App Info from GZ backup'
                tsv(report_folder, data_headers5, data_list5, tsvname)

                tlactivity = 'Powerlog App Info from GZ backup'
                timeline(report_folder, tlactivity, data_list5, data_headers5)

        if version.parse(iOSversion) >= version.parse("11"):
            if len(data_list6) > 0:
                report = ArtifactHtmlReport('Powerlog Backup Info')
                report.start_artifact_report(report_folder, 'Backup Info')
                report.add_script()
                data_headers6 = (
                    'Timestamp',
                    'Start',
                    'End',
                    'State',
                    'Finished',
                    'Has error',
                )
                report.write_artifact_data_table(data_headers6, data_list6, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Backup Info from GZ backup'
                tsv(report_folder, data_headers6, data_list6, tsvname)

                tlactivity = 'Powerlog Backup Info from GZ backup'
                timeline(report_folder, tlactivity, data_list6, data_headers6)

        if version.parse(iOSversion) >= version.parse("11"):
            if len(data_list7) > 0:
                report = ArtifactHtmlReport('Powerlog Deleted Apps')
                report.start_artifact_report(report_folder, 'Deleted Apps')
                report.add_script()
                data_headers7 = (
                    'App Deleted Date',
                    'Timestamp',
                    'App Name',
                    'App Executable Name',
                    'Bundle ID',
                    'App Build Version',
                    'App Bundle Version',
                    'App Type',
                )
                report.write_artifact_data_table(data_headers7, data_list7, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Deleted Apps from GZ backup'
                tsv(report_folder, data_headers7, data_list7, tsvname)

                tlactivity = 'Powerlog Deleted Apps from GZ backup'
                timeline(report_folder, tlactivity, data_list7, data_headers7)

        if version.parse(iOSversion) == version.parse("10"):
            if len(data_list7) > 0:
                report = ArtifactHtmlReport('Powerlog Deleted Apps')
                report.start_artifact_report(report_folder, 'Deleted Apps')
                report.add_script()
                data_headers7 = (
                    'App Deleted Date',
                    'Timestamp',
                    'App Name',
                    'App Executable Name',
                    'Bundle ID',
                    'App Build Version',
                    'App Bundle Version',
                )
                report.write_artifact_data_table(data_headers7, data_list7, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Deleted Apps from GZ backup'
                tsv(report_folder, data_headers7, data_list7, tsvname)

                tlactivity = 'Powerlog Deleted Apps from GZ backup'
                timeline(report_folder, tlactivity, data_list7, data_headers7)

        if version.parse(iOSversion) == version.parse("9"):
            if len(data_list7) > 0:
                report = ArtifactHtmlReport('Powerlog Deleted Apps')
                report.start_artifact_report(report_folder, 'Deleted Apps')
                report.add_script()
                data_headers7 = (
                    'App Deleted Date',
                    'Timestamp',
                    'App Name',
                    'Bundle ID',
                )
                report.write_artifact_data_table(data_headers7, data_list7, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Deleted Apps from GZ backup'
                tsv(report_folder, data_headers7, data_list7, tsvname)

                tlactivity = 'Powerlog Deleted Apps from GZ backup'
                timeline(report_folder, tlactivity, data_list7, data_headers7)

        if version.parse(iOSversion) >= version.parse("10"):
            if len(data_list11) > 0:
                report = ArtifactHtmlReport('Powerlog Paired Device Configuration')
                report.start_artifact_report(
                    report_folder, 'Paired Device Configuration'
                )
                report.add_script()
                data_headers11 = (
                    'Timestamp',
                    'Build',
                    'Device',
                    'HW Model',
                    'Pairing ID',
                )
                report.write_artifact_data_table(
                    data_headers11, data_list11, file_found
                )
                report.end_artifact_report()

                tsvname = 'Powerlog Paired Device Conf from GZ backup'
                tsv(report_folder, data_headers11, data_list11, tsvname)

                tlactivity = 'Powerlog Paired Device Configuration from GZ backup'
                timeline(report_folder, tlactivity, data_list11, data_headers11)
        else:
            if len(data_list11) > 0:
                report = ArtifactHtmlReport('Powerlog Paired Device Configuration')
                report.start_artifact_report(
                    report_folder, 'Paired Device Configuration'
                )
                report.add_script()
                data_headers11 = ('Timestamp', 'Build', 'Device')
                report.write_artifact_data_table(
                    data_headers11, data_list11, file_found
                )
                report.end_artifact_report()

                tsvname = 'Powerlog Paired Device Conf from GZ backup'
                tsv(report_folder, data_headers11, data_list11, tsvname)

                tlactivity = 'Powerlog Paired Device Configuration from GZ backup'
                timeline(report_folder, tlactivity, data_list11, data_headers11)

        if version.parse(iOSversion) >= version.parse("9"):
            if len(data_list15) > 0:
                report = ArtifactHtmlReport('Powerlog Torch')
                report.start_artifact_report(report_folder, 'Torch')
                report.add_script()
                data_headers15 = (
                    'Adjusted Timestamp',
                    'Bundle ID',
                    'Status',
                    'Original Torch Timestamp',
                    'Offset Timestamp',
                    'Time Offset',
                )
                report.write_artifact_data_table(
                    data_headers15, data_list15, file_found
                )
                report.end_artifact_report()

                tsvname = 'Powerlog Torch from GZ backup'
                tsv(report_folder, data_headers15, data_list15, tsvname)

                tlactivity = 'Powerlog Torch from GZ backup'
                timeline(report_folder, tlactivity, data_list15, data_headers15)

        if version.parse(iOSversion) >= version.parse("9"):
            if len(data_list18) > 0:
                report = ArtifactHtmlReport('Powerlog WiFi Network Connections')
                report.start_artifact_report(report_folder, 'WiFi Network Connections')
                report.add_script()
                data_headers18 = (
                    'Adjusted Timestamp',
                    'Current SSID',
                    'Current Channel',
                    'Offset Timestamp',
                    'Time Offset',
                )
                report.write_artifact_data_table(
                    data_headers18, data_list18, file_found
                )
                report.end_artifact_report()

                tsvname = 'Powerlog Wifi Network Connections from GZ backup'
                tsv(report_folder, data_headers18, data_list18, tsvname)

                tlactivity = 'Powerlog Wifi Network Connections from GZ backup'
                timeline(report_folder, tlactivity, data_list18, data_headers18)



import gzip
import os
import re

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import is_platform_windows, kmlgen, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class TileApp(ab.Artifact):
    _name = 'Tile App Geolocation Logs'
    _search_dirs = '*private/var/mobile/Containers/Data/Application/*/Library/log/com.thetxleapp.tile*'
    _category = 'Locations'

    def get(self, files_found, seeker):
        data_list = []

        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('gz'):
                x = gzip.open
            elif file_found.endswith('log'):
                x = open

            counter = 0
            with x(
                file_found,
                'rt',
            ) as f:
                for line in f:
                    regexdate = r"\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2}.\d{3}"
                    datestamp = re.search(regexdate, line)
                    counter += 1
                    if datestamp != None:
                        datestamp = datestamp.group(0)
                        regexlatlong = r"<[-+]?([1-8]?\d(\.\d+)?|90(\.0+)?),\s*[-+]?(180(\.0+)?|((1[0-7]\d)|([1-9]?\d))(\.\d+)?)>"
                        latlong = re.search(regexlatlong, line)
                        if latlong != None:
                            latlong = latlong.group(0)
                            latlong = latlong.strip('<')
                            latlong = latlong.strip('>')
                            lat, longi = latlong.split(',')
                            head_tail = os.path.split(file_found)
                            data_list.append(
                                (
                                    datestamp,
                                    lat.lstrip(),
                                    longi.lstrip(),
                                    counter,
                                    head_tail[1],
                                )
                            )

        if len(data_list) > 0:
            description = 'Tile app log recorded langitude and longitude coordinates.'
            report = ArtifactHtmlReport('Locations')
            report.start_artifact_report(
                report_folder, 'Tile App Geolocation Logs', description
            )
            report.add_script()
            data_headers = (
                'Timestamp',
                'Latitude',
                'Longitude',
                'Row Number',
                'Source File',
            )
            report.write_artifact_data_table(data_headers, data_list, head_tail[0])
            report.end_artifact_report()

            tsvname = 'Tile App Lat Long'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Tile App Lat Long'
            timeline(report_folder, tlactivity, data_list, data_headers)

            kmlactivity = 'Tile App Lat Long'
            kmlgen(report_folder, kmlactivity, data_list, data_headers)



from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv
from helpers.db import open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class DataUsageA(ab.Artifact):

    _name = 'Data Usage'
    _search_dirs = '**/DataUsage.sqlite'
    _category = 'Data Usage'
    _web_icon = WebIcon.WIFI

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        select
        datetime(zprocess.ztimestamp + 978307200, 'unixepoch'),
        datetime(zprocess.zfirsttimestamp + 978307200, 'unixepoch'),
        datetime(zliveusage.ztimestamp + 978307200, 'unixepoch'),
        zbundlename,
        zprocname,
        zwifiin,
        zwifiout,
        zwwanin,
        zwwanout
        from zliveusage, zprocess
        where
        zprocess.z_pk = zliveusage.zhasprocess
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                data_list.append(
                    (
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                        row[7],
                        row[8],
                    )
                )

            report = ArtifactHtmlReport('Data Usage')
            report.start_artifact_report(report_folder, 'Data Usage')
            report.add_script()
            data_headers = (
                'Process Timestamp',
                'Process First Timestamp',
                'Live Usage Timestamp',
                'Bundle ID',
                'Process Name',
                'WIFI In',
                'WIFI Out',
                'WWAN IN',
                'WWAN Out',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Data Usage'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Data Usage'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Data Usage available')

        db.close()
